/*****************************************************************************
 * Copyright (c) 2014, 2016 CEA LIST, Christian W. Damus, and others.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  CEA LIST - Initial API and implementation
 *  Christian W. Damus - bug 474481
 *****************************************************************************/
package org.eclipse.papyrusrt.umlrt.core.types.advice;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.gmf.runtime.common.core.command.ICommand;
import org.eclipse.gmf.runtime.emf.type.core.commands.SetValueCommand;
import org.eclipse.gmf.runtime.emf.type.core.edithelper.AbstractEditHelperAdvice;
import org.eclipse.gmf.runtime.emf.type.core.requests.IEditCommandRequest;
import org.eclipse.gmf.runtime.emf.type.core.requests.SetRequest;
import org.eclipse.papyrusrt.umlrt.core.utils.CapsuleUtils;
import org.eclipse.papyrusrt.umlrt.core.utils.NewElementUtil;
import org.eclipse.papyrusrt.umlrt.profile.UMLRealTime.CapsulePart;
import org.eclipse.uml2.uml.Classifier;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;
import org.eclipse.uml2.uml.UMLPackage;

import com.google.common.base.Strings;

/**
 * Advice for {@link CapsulePart} on the setting of its
 * {@link UMLPackage.Literals#TYPED_ELEMENT__TYPE type}.
 */
public class CapsulePartSetTypeAdvice extends AbstractEditHelperAdvice {

	public CapsulePartSetTypeAdvice() {
		super();
	}

	@Override
	public boolean approveRequest(IEditCommandRequest request) {
		// check that the type of the CapsulePart is a capsule, and nothing else
		if (request instanceof SetRequest) {
			SetRequest setRequest = (SetRequest) request;
			EStructuralFeature feature = setRequest.getFeature();
			if (UMLPackage.eINSTANCE.getTypedElement_Type().equals(feature)) {
				// new value should be a protocol or null
				Object newValue = ((SetRequest) request).getValue();
				if (newValue instanceof Classifier) {
					if (CapsuleUtils.isCapsule((Classifier) newValue)) {
						return true;
					} else {
						return false;
					}
				} else {
					return false;
				}
			}
			return super.approveRequest(setRequest);

		}
		return super.approveRequest(request);
	}

	/**
	 * React to the setting of the initial type of an unnamed capsule-part by
	 * initializing its name.
	 */
	@Override
	protected ICommand getAfterSetCommand(SetRequest request) {
		ICommand result = null;
		EStructuralFeature feature = request.getFeature();

		if (UMLPackage.eINSTANCE.getTypedElement_Type().equals(feature)) {
			// if element is unnamed or is itself a new element, set a name
			// åaccording to the new Type name
			Object newValue = request.getValue();
			EObject elementToEdit = request.getElementToEdit();
			if ((newValue instanceof Type)
					&& (elementToEdit instanceof Property)
					&& (Strings.isNullOrEmpty(((Property) elementToEdit).getName())
							|| NewElementUtil.isCreatedElement(elementToEdit))) {

				Property property = (Property) elementToEdit;
				String name = ((Type) newValue).getName();
				if (!Strings.isNullOrEmpty(name)) {
					String newName = NewElementUtil.getUniqueName(property, (Type) newValue);
					result = new SetValueCommand(new SetRequest(request.getElementToEdit(), UMLPackage.eINSTANCE.getNamedElement_Name(), newName));
				}
			}
		}

		return result;
	}

}
